package net.java.amateras.uml.activitydiagram.editpart;

import java.util.Collections;
import java.util.List;

import net.java.amateras.uml.activitydiagram.model.VerticalPartitionModel;
import net.java.amateras.uml.figure.EntityFigure;
import net.java.amateras.uml.model.AbstractUMLEntityModel;
import net.java.amateras.uml.model.RootModel;

import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.draw2d.RectangleFigure;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.draw2d.geometry.Insets;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.NonResizableEditPolicy;
import org.eclipse.gef.editpolicies.XYLayoutEditPolicy;
import org.eclipse.gef.requests.CreateRequest;

/**
 * 
 * @author Naoki Takezoe
 */
public class VerticalPartitionEditPart extends AbstractActivityEntityEditPart {

	protected IFigure createFigure() {
		VarticalPartitionFigure figure = new VarticalPartitionFigure();
		figure.updatePresentation((VerticalPartitionModel)getModel());
		return figure;
	}
	
	protected void refreshVisuals() {
		super.refreshVisuals();
		VarticalPartitionFigure figure = (VarticalPartitionFigure)getFigure();
		figure.updatePresentation((VerticalPartitionModel)getModel());
	}
	
	protected void createEditPolicies() {
		installEditPolicy(EditPolicy.LAYOUT_ROLE, new RootEditPolicy());
	}	
	
	protected List getModelChildren() {
		return Collections.EMPTY_LIST;
	}
	
	private class VarticalPartitionFigure extends RectangleFigure implements EntityFigure {
		
		private Label label = new Label();
		
		public VarticalPartitionFigure(){
			setLayoutManager(new XYLayout());
			label.setBorder(new LineBorder(){
				public void paint(IFigure figure, Graphics graphics, Insets insets) {
					tempRect.setBounds(getPaintRectangle(figure, insets));
					if (getWidth() % 2 == 1) {
						tempRect.width--;
						tempRect.height--;
					}
					tempRect.shrink(getWidth() / 2, getWidth() / 2);
					graphics.setLineWidth(getWidth());
					if (getColor() != null)
						graphics.setForegroundColor(getColor());
					graphics.drawLine(tempRect.x, tempRect.y + tempRect.height, 
							tempRect.x + tempRect.width, tempRect.y + tempRect.height);
				}
			});
			
			add(label);
			setOpaque(false);
			setFill(false);
		}
		
		public void updatePresentation(VerticalPartitionModel model){
			this.label.setText(model.getPartitionName());
			setConstraint(this.label, new Rectangle(0, 0, 
					model.getConstraint().width, 20));
		}

		public Rectangle getCellEditorRectangle() {
			return label.getBounds().getCopy();
		}

		public Label getLabel() {
			return this.label;
		}
	}
	
	/** GfBbg|V[ */
	private class RootEditPolicy extends XYLayoutEditPolicy {

		protected EditPolicy createChildEditPolicy(EditPart child) {
			return new NonResizableEditPolicy();
		}
		
		protected Command createAddCommand(EditPart child, Object constraint) {
			if (!(child.getModel() instanceof AbstractUMLEntityModel)) {
				return null;
			}
			CreateAddCommand command = new CreateAddCommand();
			command.setModel((AbstractUMLEntityModel) child.getModel());
			command.setTarget((RootModel)((VerticalPartitionModel)getModel()).getParent());
			
			ChangeConstraintCommand nextCommand = new ChangeConstraintCommand();
			nextCommand.setModel((AbstractUMLEntityModel)child.getModel());
			
			Rectangle rectangle = (Rectangle)constraint;
			VerticalPartitionModel model = (VerticalPartitionModel)getModel();
			rectangle.x = rectangle.x + model.getConstraint().x;
			rectangle.y = rectangle.y + model.getConstraint().y;
			
			nextCommand.setConstraint(rectangle);
			
			return command.chain(nextCommand);
		}
		
		protected Command createChangeConstraintCommand(EditPart child,Object constraint) {
			ChangeConstraintCommand command = new ChangeConstraintCommand();
			command.setModel((AbstractUMLEntityModel)child.getModel());
			command.setConstraint((Rectangle)constraint);
			return command;
		}
		
		protected Command getCreateCommand(CreateRequest request) {
			if(!(getHost().getModel() instanceof RootModel)){
				return null;
			}
			CreateCommand command = new CreateCommand();
			
			Rectangle constraint = (Rectangle) getConstraintFor(request);
			VerticalPartitionModel parentModel = (VerticalPartitionModel)getModel();
			constraint.x = constraint.x + parentModel.getConstraint().x;
			constraint.y = constraint.y + parentModel.getConstraint().y;
			
			AbstractUMLEntityModel model = (AbstractUMLEntityModel) request.getNewObject();
//			if (!RESIZABLE.contains(model.getClass())) {
				constraint.width = -1;
				constraint.height = -1;
//			}
			model.setConstraint(constraint);
			
			command.setRootModel(getHost().getModel());
			command.setModel(model);
			return command;
		}
		
		protected Command getDeleteDependantCommand(Request request) {
			return null;
		}
	}
	
	/** ̕ύXR}h */
	private class ChangeConstraintCommand extends Command {
		
		private AbstractUMLEntityModel model;
		private Rectangle constraint;
		private Rectangle oldConstraint;
		
		public void execute(){
			model.setConstraint(constraint);
		}
		
		public void setConstraint(Rectangle constraint){
			this.constraint = constraint;
		}
		
		public void setModel(AbstractUMLEntityModel model){
			this.model = model;
			oldConstraint = model.getConstraint();
		}
		
		public void undo() {
			model.setConstraint(oldConstraint);
		}
	}
	
	/** f̐VK쐬R}h */
	private class CreateCommand extends Command {
		
		private RootModel root;
		private AbstractUMLEntityModel model;
		
		public void execute() {
			root.copyFilter(model);
			root.copyPresentation(model);
			root.addChild(model);
		}
		
		public void setRootModel(Object root) {
			this.root = (RootModel)root;
		}
		
		public void setModel(Object model) {
			this.model = (AbstractUMLEntityModel) model;
		}
		
		public void undo() {
			root.removeChild(model);
		}
	}

	private class CreateAddCommand extends Command {
		private RootModel target;
		private AbstractUMLEntityModel model;
		private AbstractUMLEntityModel container;
		
		public void execute() {
			this.container = model.getParent();
			container.removeChild(model);
			target.addChild(model);
		}
		
		public void undo() {
			target.removeChild(model);
			container.addChild(model);
		}
		public void setTarget(RootModel target) {
			this.target = target;
		}
		
		public void setModel(AbstractUMLEntityModel model) {
			this.model = model;
		}
	}

}
